-- All Event Listeners to go here. Everything else should be loaded at this point

SKG:AddEventListener( "Blizzard", "ADDON_LOADED" )
SKG:AddEventListener( "Blizzard", "LOOT_OPENED" )
SKG:AddEventListener( "Blizzard", "RAID_ROSTER_UPDATE" )
SKG:AddEventListener( "Blizzard", "CHAT_MSG_WHISPER" )

SKG:AddEventListener( "LibRockEvent-1.0", "FullyInitialized" )

-- Setup filters on inbound/outbound whispers for SKG stuff
-- Note, if you're running WIM, this won't work (yet)
function SKG_WhisperFilter( msg )
	-- inbound stuff
	-- These are safe to screen out
	if ( ( msg == "bid" ) or
		 ( msg == "retract" ) or
		 ( msg == "suicide" ) or
		 ( msg == "skg" ) ) then
		return true
	end
	-- outbound stuff
	-- don't do this. Requestor will not see any responses
--[[
	if ( strsub( msg, 1, 3 ) == "SKG" ) then
		return true
	end
]]--
end

ChatFrame_AddMessageEventFilter( "CHAT_MSG_WHISPER", SKG_WhisperFilter )

-- Other events to worry about
-- PARTY_LOOT_METHOD_CHANGED
SKG.CHAT_MSG_WHISPER = function( self, value, namespace, event )
	-- This is here for backwards compatibility to the old way of
	-- doing things, and is necessary if the users in the raid are
	-- not running the new version of the addon at all
	local cmd = strlower( arg1 )
	local sender = arg2
	if ( cmd == "bid" ) then
		-- bid handling code
		SKG_Debug( "Received bid request from " .. sender )
		if ( SKG.CurrBidList == "" ) then
			-- I have no idea there's a bidding event going on. Msg him back
			SendChatMessage( SKGL["SKG_ERROR_NOBIDEVENT"], "WHISPER", nil, sender )
		else
			-- We think there's a bid going on ...
			-- Are we in the same raid?
			if ( #SKG.Raiders ~= 0 ) then
				-- we're in a raid, is the sender in our raid?
				local found = 0
				for i = 1, #SKG.Raiders do
					if ( SKG.Raiders[i].name == sender ) then
						-- he's in our raid
						found = i
						break
					end
				end
				if ( found == 0 ) then
					SendChatMessage( SKGL["SKG_ERROR_NOTINMYRAID"], "WHISPER", nil, sender )
				else
					local inList = 0
					local db = SKG.db.profile.RollLists[SKG.CurrBidList]
					for i = 1, #db do
						if ( db[i] == SKG.Raiders[found].mPID ) then
							inList = i
							break
						end
					end
					if ( inList == 0 ) then
						SendChatMessage( SKGL["SKG_ERROR_NOTINBIDLIST"], "WHISPER", nil, sender )
						SKG_Print( SKGL["SKG_ERROR_INVALIDBIDDER"] .. " " .. sender )
					else
						-- Everything looks good. Send the bid event
						SKG:SendCommMessage( "GROUP", "BIDBD", sender )
					end
				end
			end
		end
	elseif ( cmd == "suicide" ) then
		-- report back the current list
		SKG_Debug( "Received suicide request from " .. sender )
		-- if I know about a bid event, go ahead and respond
		if ( SKG.CurrBidList ~= "" ) then
			SendChatMessage( SKGL["SKG_MSG_BIDDERHEADER"], "WHISPER", nil, sender )

			local db = SKG.db.profile.RollLists[SKG.CurrBidList]
			local numEntries = 0
			local outString = ""
			-- loop through current bid list
			for i = 1, #db do
				-- if this person is in our raid, add them into the output string
				for j = 1, #SKG.Raiders do
					if ( db[i] == SKG.Raiders[j].mPID ) then
						if ( outString == "" ) then
							outString = "<" .. i .. "> " .. SKG.Raiders[j].name
						else
							outString = outString .. ", <" .. i .. "> " .. SKG.Raiders[j].name
						end
						numEntries = numEntries + 1
						break
					end
				end

				if ( numEntries == 5 ) then
					SendChatMessage( outString, "WHISPER", nil, sender )
					numEntries = 0
					outString = ""
				end
			end
			if ( outString ~= 0 ) then
				SendChatMessage( outString, "WHISPER", nil, sender )
			end
		else
			SendChatMessage( SKGL["SKG_ERROR_NOBIDEVENT"], "WHISPER", nil, sender )
		end
	elseif ( cmd == "retract" ) then
		-- bid retraction code
		SKG_Debug( "Received retract request from " .. sender )
		if ( SKG.CurrBidList == "" ) then
			-- I have no idea there's a bidding event going on. Msg him back
			SendChatMessage( SKGL["SKG_ERROR_NOBIDEVENT"], "WHISPER", nil, sender )
		else
			-- We think there's a bid going on ...
			-- Are we in the same raid?
			if ( #SKG.Raiders ~= 0 ) then
				-- we're in a raid, is the sender in our raid?
				local found = 0
				for i = 1, #SKG.Raiders do
					if ( SKG.Raiders[i].name == sender ) then
						-- he's in our raid
						found = i
						break
					end
				end
				if ( found == 0 ) then
					SendChatMessage( SKGL["SKG_ERROR_NOTINMYRAID"], "WHISPER", nil, sender )
				else
					local inList = 0
					local db = SKG.db.profile.RollLists[SKG.CurrBidList]
					for i = 1, #db do
						if ( db[i] == SKG.Raiders[found].mPID ) then
							inList = i
							break
						end
					end
					if ( inList == 0 ) then
						SendChatMessage( SKGL["SKG_ERROR_NOTINBIDLIST"], "WHISPER", nil, sender )
						SKG_Print( SKGL["SKG_ERROR_INVALIDBIDDER"] .. " " .. sender )
					else
						-- Everything looks good. Send the bid event
						SKG:SendCommMessage( "GROUP", "BIDRT", sender )
					end
				end
			end
		end
	elseif ( cmd == "skg" ) then
		SendChatMessage( SKGL["SKG_MSG_SKGINST1"], "WHISPER", nil, sender )
		SendChatMessage( SKGL["SKG_MSG_SKGINST2"], "WHISPER", nil, sender )
		SendChatMessage( SKGL["SKG_MSG_SKGINST3"], "WHISPER", nil, sender )
	end
end

SKG.RAID_ROSTER_UPDATE = function ( self, value, namespace, event )
	SKG_Debug( "Received RAID_ROSTER_UPDATE" )
	local numRaiders = GetNumRaidMembers()
	if ( numRaiders > 0 ) then
		-- toggle main variables
		SKG.inRaid = true
		SKG_UpdateRaiders()

		-- toggle buttons, show panels, etc
		-- this will be handled when a loot event occurs...
		--[[ 
		if ( SKG_FRAME_STATE == "Bid" ) then
			SKG_BidButtonFrame:Show()
		end ]]--

		-- if you also happen to be raid leader, enable these:
		if ( IsRaidLeader() == 1 ) then
			SKG_BF_LootTypeButton:Show()
			SKG_BF_LootThreshButton:Show()
			SKG_BF_LootTypeButton:Enable()
			SKG_BF_LootThreshButton:Enable()
			-- and fill in the ML assignment dewdrop
			SKG.lootTypeSelections.args[3].args = {}
			for i = 1, #SKG.Raiders do
				raidName = SKG.Raiders[i].name
				raidIndex = SKG.Raiders[i].raidID
				SKG_Debug( "n = " .. raidName .. ", rI = " .. raidIndex )
				local a = {
					type = "execute",
					name = raidName,
					desc = raidName,
					func = function( value )
						-- Set Master Looter to this player
						SetLootMethod( "master", value )
						Dewdrop:Close()
					end,
					passValue = raidName,
					order = raidIndex,
				}
				table.insert( SKG.lootTypeSelections.args[3].args, a )
			end
		else
			SKG_BF_LootTypeButton:Hide()
			SKG_BF_LootThreshButton:Hide()
		end
	else
		SKG.inRaid = false
		SKG.Raiders = {}
		-- hide buttons/panels, etc
		if ( SKG_FRAME_STATE == "Bid" ) then
			SKG_BidButtonFrame:Hide()
		end
		SKG_BF_LootTypeButton:Disable()
		SKG_BF_LootThreshButton:Disable()
	end
	SKG_UpdateCurrentList()
end

SKG.ADDON_LOADED = function( self, value, namespace, event )
	if ( arg1 == "SKG" ) then
		local db = SKG.db.profile
		SKG_Debug( "Processing ADDON_LOADED event for SKG" )
		-- We're safe to do things like process saved variables here.
		-- See the header of SKG_Globals.lua for a full reference of
		-- possible values

		-- Call InitAllPanels to make sure we have the right values
		SKG_InitAllPanels()

		-- Testing out comms...
		SKG:SendCommMessage( "GUILD", "DEBUG", "test message" )
	end
end

SKG.LOOT_OPENED = function( self, value, namespace, event )
	-- The player who is the master looter should get the goods
	SKG_Debug( "Received LOOT_OPENED event" )

	-- Note: This ASSUMES that looting something forces you to target the mob you're looting
	local mobGUID = UnitGUID("target")

	-- Only process anything if we're in a raid
	if not ( GetNumRaidMembers() == 0 ) then
		-- we're in a raid
		-- Only do stuff if we're the Loot Master
		SKG_Debug( "We're in a raid" )
		local myRaidID = SKG_GetMyRaidID()
		local lootMethod, MLPartyID, MLRaidID = GetLootMethod()
		if ( lootMethod == "master" ) then
			SKG_Debug( "Master Looter method" )
			if ( MLRaidID == myRaidID ) then
				-- we're the ML, do stuff
				SKG_Debug( "I'm the ML" )
				-- Send a LOOTC event so everyone's loot lists get cleared
				-- Actually, I'm gonna remove this. People should be clearing out
				-- items that they don't want handled in a loot event by hand.
				-- SKG:SendCommMessage( "GROUP", "LOOTC", "" )

				-- Iterate through the looted items, see which ones match out threshold
				local LootThreshold = GetLootThreshold()
				SKG_Debug( "threshold = " .. LootThreshold )
				local index = 1
				for index = 1, GetNumLootItems(), 1 do
					if ( LootSlotIsItem(index) ) then
						local itemLink = GetLootSlotLink( index );
						local retVal = SKG_ParseItem( itemLink, "IDRA" )
						local itemID, itemRarity = unpack( retVal, 1, 2 )

						if ( SKG_LootIgnore[itemID] ) then
							-- this is an item we're ignoring. Don't do anything
						else
							-- check our loot threshold
							if ( itemRarity >= LootThreshold ) then
								-- One final check. Is the item in question already in our
								-- biddable items list:
								local found = false
								for i = 1, #SKG.LootItems do
									if ( ( SKG.LootItems[i].mGUID == mobGUID ) and
										 ( SKG.LootItems[i].link == itemLink ) ) then
										found = true
										break
									end
								end

								if ( found == false ) then
									local payload = { mobGUID, itemLink }
									SKG:SendCommMessage( "GROUP", "LOOTI", payload )
								else
									-- do nothing. We already have this item in our loot table
								end
							end
						end
					end
				end
			else
				SKG_Debug( "I'm not the ML, myrid = " .. myRaidID .. ", value = " .. MLRaidID )
			end
		else
			SKG_Debug( "Master Looter isn't the method, " .. lootMethod )
		end
	else
		SKG_Debug( "We're not in a raid" )
	end
end

SKG.FullyInitialized = function( self, value, namespace, event )
	if ( SKG_INIT_GUILD ) then
		local db = SKG.db.profile.Options.Security
		-- Some values just don't seem to be present even after all the regular addon
		-- processing has occurred. So lets try this, after the user is already in, and
		-- has opted to open the main frame...

		SKG_INIT_GUILD = false

		SKG_SetSecurityLevel()

		SKG.me = UnitName( "player" )

		-- Now we can assign the ranks in the security chooser
		for index = 1, GuildControlGetNumRanks() do
			local t = {
				type = "execute",
				name = GuildControlGetRankName(index),
				desc = GuildControlGetRankName(index),
				func = function( value )
					SKG_SF_CurrentRankText:SetText(
						SKGL["SKG_LBL_SF_CURRENTRANK_TEXT"] .. GuildControlGetRankName(value) )
					db.AuthRank = value
					Dewdrop:Close()
				end,
				passValue = index,
				order = 100 + index,
			}
			tinsert( SKG.RankChooserOptions.args, t )
		end

		if ( db.RankBasedAuth ) then
			SKG_SF_RankBasedCB:SetChecked( true )
			SKG_SF_ManualAuthCB:SetChecked( false )
			SKG_SF_RankChooser:Enable()
			SKG_EnableNormalText( SKG_SF_CurrentRankText )
			SKG_SF_CurrentRankText:SetText( SKGL["SKG_LBL_SF_CURRENTRANK_TEXT"] ..
				GuildControlGetRankName( db.AuthRank ) )

			SKG_SF_AuthListHover:Disable()
		else
			SKG_SF_RankBasedCB:SetChecked( false )
			SKG_SF_ManualAuthCB:SetChecked( true )
			SKG_SF_RankChooser:Disable()
			SKG_DisableNormalText( SKG_SF_CurrentRankText )

			SKG_SF_AuthListHover:Enable()
		end

		-- We should be able to fill in the "mainChooser" selector for inserting players
		SKG_UpdateMainsChooser()
		SKG_UpdateToonChooser()

		-- Fill in the guild player chooser
		-- The next sort is via class. Add each class to each rank menu
		for classIndex = 1, 9 do
			local u = {
				type = "group",
				name = SKG_ClassNames[classIndex],
				desc = SKG_ClassNames[classIndex],
				order = 100 + classIndex,
				args = { },
			}
			tinsert( SKG.GuildPlayerChooserOptions.args, u )

			-- And finally, an alphabetic grouping...
			for alphaIndex = 1, 5 do
				if ( alphaIndex == 1 ) then
					nameDesc = "A-C"
				elseif ( alphaIndex == 2 ) then
					nameDesc = "D-F"
				elseif ( alphaIndex == 3 ) then
					nameDesc = "G-L"
				elseif ( alphaIndex == 4 ) then
					nameDesc = "M-R"
				else
					nameDesc = "S-Z"
				end

				local v = {
					type = "group",
					name = nameDesc,
					desc = nameDesc,
					order = 100 + alphaIndex,
					args = { },
				}
				tinsert( SKG.GuildPlayerChooserOptions.args[classIndex].args, v )
			end
		end

		-- and now, individual players. Don't put an order in, it'll sort 'em
		-- alphabetically
		numGuildMembers = GetNumGuildMembers(true)
		for index = 1, numGuildMembers do
			local toonName, gRank = 0

			toonName, gRank, g, l, toonClass = GetGuildRosterInfo(index);			

			-- Parse out where its going to go in our tree.
			for i,v in ipairs( SKG_ClassNames ) do
				if ( v == toonClass ) then CI = i end
			end
			local AI = alphaIndex( toonName )

			local t = {
				type = "execute",
				name = "" .. toonName,
				desc = "" .. toonName,
				func = function( value )
					SKG_SelectedPlayerName = value
					SKG_SF_SelectedPlayerText:SetText(
						SKGL["SKG_LBL_SF_CURRENTPLAYER_TEXT"] .. value )
					-- Enable the selected player action buttons
					SKG_SF_AddAuthButton:Enable()
					SKG_SF_DelAuthButton:Enable()
					SKG_SF_AddProxyButton:Enable()
					Dewdrop:Close()
				end,
				passValue = toonName,
			}
			tinsert( SKG.GuildPlayerChooserOptions.args[CI].args[AI].args, t )
		end
	end

	-- Now that the full list is in there, remove entries that are empty
	for cI = 9, 1, -1 do
		for aI = 5, 1, -1 do
			if ( #SKG.GuildPlayerChooserOptions.args[cI].args[aI].args == 0 ) then
				tremove( SKG.GuildPlayerChooserOptions.args[cI].args, aI )
			end
		end
		if ( next( SKG.GuildPlayerChooserOptions.args[cI].args ) == nil ) then
			tremove( SKG.GuildPlayerChooserOptions.args, cI )
		end
	end

	-- Do the initial setup of SPlayers
	SKG_UpdateSPlayers()

	-- Attempt to detect whether the old version of SKG is loaded and if we possibly
	-- have lists to convert
	if ( SKG_Options ) then
		StaticPopup_Show( "SKG_CONFIRM_IMPORT" )
	else
		SKG_Debug("Old version not present")
	end
end
